#Base Types
EXCEPTION = "exception"
ALLOWANCE = "allowance"

ASSIGNATION = "assignation"
PROVISION = "provision"
ASSIGNATION_REQUEST = "assignation_request"
PROVISION_REQUEST = "provision_request"
ACTIVATE_POD = "activate_pod"